/*
 * File:   elekey.c
 * Author: ikedahiroaki JA7EIF
 *
 * Created on 2019/06/28, 14:24
 * Update on 2022/07/21
 * Electric Keyer version 2
 *  Fosc=8MHz
 * Serial Format 9600bps 8N1
 * 	Command+';'+Parameter+'CR'
 * 		MOD;	3=elekey 2=bugkey 1=Duplex 0=Single
 *		DOT;	0...10
 *		DAS;	0...10
 *		SPD;	0...20
 *		RAT;	0...9
*/

// PIC18F1220 Configuration Bit Settings

// 'C' source line config statements

// CONFIG1H
#pragma config OSC = INTIO1     // Oscillator Selection bits (Internal RC oscillator, CLKO function on RA6 and port function on RA7)
#pragma config FSCM = OFF       // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal External Switchover mode disabled)

// CONFIG2L
#pragma config PWRT = ON        // Power-up Timer Enable bit (PWRT enabled)
#pragma config BOR = OFF        // Brown-out Reset Enable bit (Brown-out Reset disabled)
// BORV = No Setting

// CONFIG2H
#pragma config WDT = OFF        // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config MCLRE = OFF      // MCLR Pin Enable bit (RA5 input pin enabled, MCLR disabled)

// CONFIG4L
#pragma config STVR = ON        // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF        // Low-Voltage ICSP Enable bit (Low-Voltage ICSP disabled)

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 (00200-0007FFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 (000800-000FFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot Block (000000-0001FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (00200-0007FFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (000800-000FFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0001FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (00200-0007FFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (000800-000FFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0001FFh) not protected from table reads executed in other blocks)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define DashPort        PORTBbits.RB0
#define DotPort         PORTBbits.RB2
#define DashLenPort     PORTBbits.RB3
#define SpeedDNPort     PORTBbits.RB5
#define ModePort        PORTBbits.RB6
#define SpeedUPPort     PORTBbits.RB7
#define DotLenPort      PORTAbits.RA2
#define RatioPort       PORTAbits.RA3

#define LED_Y           LATAbits.LA0
#define LED_B           LATAbits.LA7
#define KEYOUT          LATAbits.LA1
#define LED_ON          1
#define LED_OFF         0
#define KEY_ON          1
#define KEY_OFF         0
      
#define MODE_DEFAULT 	3  		//0:X[P 1:X[ 2:oOL[ 3:GL[
#define SPD_DEFAULT 	15  	//ftHgl̔zvf
#define RATIO_DEFAULT 	1   	//ftHgl̔zvf
#define DOTLEN_DEFAULT 	5   	//ftHgl̔zvf
#define DASHLEN_DEFAULT 5   	//ftHgl̔zvf
#define SLEEP_TIME		1000    //1s

//xc8ł͌^w肵ȂꍇunsignedƂ݂ȂƂ̂
__bit scounter = 0;
__bit ledoffcounter = 0;
__bit startcounter = 0;
int mseconds = 0;
int cseconds = 0;
int sleeptime = 0;

//eXCb` `^OΉϐ
char  DotSW = 0xFF;
char  DashSW = 0xFF;
char  ModeSW = 0xFF;
char  DotLenSW = 0xFF;
char  DashLenSW = 0xFF;
char  SpeedUPSW = 0xFF;
char  SpeedDNSW = 0xFF;
char  RatioSW = 0xFF;

//
int DotLen = 0;
int DashLen = 0;
int DotSpLen = 0;
int DashSpLen = 0;
			//250,240,230,220,210,200,190,180,170,160,150,140,130,120,110,100,90,80,70,60,50/
char Speed[21] = {24,25,26,27,29,30,32,33,35,38,40,43,46,50,55,60,66,75,86,100,120};    //Speed
char Ratio[10] = {28,30,32,34,36,38,40,42,44,46};
char DotLength[11] = {5,6,7,8,9,10,11,12,13,14,15};
char DashLength[11] = {5,6,7,8,9,10,11,12,13,14,15};
// SerialMobt@
unsigned char RcvBuf[20];
char RcvIdx;

// e{^̏
char Mode;
char bug_spdvalue;
char ele_spdvalue;
char ratiovalue;
char bug_dotlenvalue;
char ele_dotlenvalue;
char dashlenvalue;

// KEY̏
char DotOn = 0;
char DashOn = 0;

// eXCb`̕ω
__bit DotSWClick = 0;
__bit DashSWClick = 0;
__bit ModeSWClick = 0;
__bit DotLenSWClick = 0;
__bit DashLenSWClick = 0;
__bit SpeedDNSWClick = 0;
__bit SpeedUPSWClick = 0;
__bit RatioSWClick = 0;

// EEPROMɃf[^
int EEPROM_write(char addr, char data)
{
	int inte;
	
	inte = GIE;
   	GIE = 0;
	EEADR = addr;
	EEDATA = data;
	EECON1bits.EEPGD = 0;
	EECON1bits.CFGS = 0;
	EECON1bits.WREN = 1;
	EECON2 = 0x55;
	EECON2 = 0xaa;
	EECON1bits.WR = 1;
    while(WR);
   	GIE = inte;
	EECON1bits.WREN = 0;
    return EECON1bits.WRERR;
}

// EEPROMf[^ǂ
char EEPROM_read(char addr)
{
	EEADR = addr;
	EECON1bits.EEPGD = 0;
	EECON1bits.CFGS = 0;
	EECON1bits.RD = 1;
	return EEDATA;
}

// EEPROMɃftHgl
void set_default_value(void)
{
	Mode = MODE_DEFAULT;
	bug_spdvalue = SPD_DEFAULT;
	ele_spdvalue = SPD_DEFAULT;
	bug_dotlenvalue = DOTLEN_DEFAULT;
	ele_dotlenvalue = DOTLEN_DEFAULT;
	ratiovalue = RATIO_DEFAULT;
	dashlenvalue = DASHLEN_DEFAULT;
	EEPROM_write(0, 0x10);
	EEPROM_write(1, Mode);
	EEPROM_write(2, bug_spdvalue);
	EEPROM_write(3, ele_spdvalue);
	EEPROM_write(4, bug_dotlenvalue);
	EEPROM_write(5, ele_dotlenvalue);
	EEPROM_write(6, ratiovalue);
	EEPROM_write(7, dashlenvalue);
}

// FLED_
void LED_B_on(void)
{
	LED_B = LED_ON;
	ledoffcounter = 1;
}

// FLED_
void LED_Y_on(void)
{
	LED_Y = LED_ON;
	ledoffcounter = 1;
}

// X[v[hɓ
void goto_sleep(void)
{
	IDLEN = 0;
    SCS1 = 0;
    SCS0 = 0;
   	SLEEP();
}

// write to memory sprintf̓TCY傫̂ŎgpȂ
void WriteToMem(unsigned char * mem, unsigned char * moji)
{
    while(*moji)
        *mem++ = *moji++;
    *mem = '\0'; 
}

// VAM
void SerialWrite(unsigned char *s, int len)
{
    int i;

	for(i = 0; i < len; i++){
        TXREG = *s++;
        while(TRMT == 0);
    }
}

// MR}h̎s
void DoCommand(char *cmd, char *param)
{
	unsigned int p, s;
	unsigned char c[10];
	// test
	// SerialWrite("TEST\n");
    // command 
    // MOD; 3=elekey 2=bugkey 1=Duplex 0=Single
    // DOT;	0...10
    // DAS;	0...10
    // SPD;	0...20
    // RAT;	0...9
    p = (unsigned)atoi(param);
	//sprintf(c,"OK\n");
//    SerialWrite(cmd);
    WriteToMem(c, "NG\n");
    if(strcmp(cmd, "MOD") == 0){      
        if(p < 4){
            Mode = p;
        	EEPROM_write(1, p);
            WriteToMem(c, "OK\n");
        }
    }
    else if(strcmp(cmd, "DOT") == 0){      
        if(p < 11){
            if(Mode == 3){
                ele_dotlenvalue = p;
                EEPROM_write(5, p);
                WriteToMem(c, "OK\n");
            }
            else if(Mode == 2){
                bug_dotlenvalue = p;
                EEPROM_write(4, p);
                WriteToMem(c, "OK\n");
            }
        }
    }
    else if(strcmp(cmd, "DAS") == 0){      
        if(p < 11){
            if(Mode == 3){
                dashlenvalue = p;
                EEPROM_write(7, p);
                WriteToMem(c, "OK\n");
            }
        }
    }
    else if(strcmp(cmd, "SPD") == 0){      
        if(p < 21){
            if(Mode == 3){
                ele_spdvalue = p;
   				EEPROM_write(3, p);
                WriteToMem(c, "OK\n");
            }
            else if(Mode == 2){
                bug_spdvalue = p;
   				EEPROM_write(2, p);
                WriteToMem(c, "OK\n");
            }
        }
    }
    else if(strcmp(cmd, "RAT") == 0){      
        if(p < 10){
            if(Mode == 3){
                ratiovalue = p;
                EEPROM_write(6, p);
                WriteToMem(c, "OK\n");
    			SerialWrite(c, 3);
            }
        }
    }
    else if(strcmp(cmd, "INF") == 0){      
		//sprintf(c, "MOD;%d\n", Mode); //tH[}b^[̓sŎgpłȂ
        // tH[}bg
        // 0  [h 0=Single 1=Double 2=Bug 3= Ele
        // 1  Bugkey Speed 0..20
        // 2  Bugkey Dotlength 0..10
        // 3  Elekey Speed 0..20
        // 4  Elekey Ratio 0..9
        // 5  Elekey Dotlength 0..10
        // 6  Elekey Dashlength 0..10
        c[0] = Mode;
        c[1] = bug_spdvalue;
        c[2] = bug_dotlenvalue;
        c[3] = ele_spdvalue;
        c[4] = ratiovalue;
        c[5] = ele_dotlenvalue;
        c[6] = dashlenvalue;
        c[7] = 255;
        c[8] = 0;
    	SerialWrite(c, 8);
    }
    else{
        WriteToMem(c, "?\n");
    	SerialWrite(c, 2);
    }

// test
//	SerialWrite(cmd);
//	SerialWrite("\n");
//	SerialWrite(param);
//	SerialWrite("\n");
}

// VAM
void Recived(void)
{
    unsigned int i, d;
	char c;
    unsigned char cmd[4], param[3];
	
	c = RCREG;
	if(c == 0xd){
        cmd[0] = 0;
        param[0] = 0;
        // R}h3+';'
		d = 0;
        for(i=0; i < RcvIdx; i++){
            if(RcvBuf[i] == ';'){       // cpmmand delimiter
                cmd[i] = 0;
                d = i;
                break; 
            }
            else{
                if(i < 5)
                    cmd[i] = RcvBuf[i];
                else
                    cmd[0] = 0;
            }
        }
        if(d != 3)
            cmd[0] = 0;
        // p[^
		for(i=d+1; i < RcvIdx; i++){
			if((i-d-1) < 4)
                param[i-d-1] = RcvBuf[i];
        }
		if((i-d-1) < 4)
            param[i-d-1] = 0;
        else
            param[0] = 0;
        //SerialWrite(cmd);	
        DoCommand(cmd, param);
		RcvBuf[0] = 0;
        RcvIdx = 0;
	}
	else{
		if(0x1F < c)
            RcvBuf[RcvIdx++] = c;
        if(19 < RcvIdx)
			RcvIdx = 0;
	}
}

//Z_̒ݒ肷
void DotLenSet(void)
{
	if(Mode ==2){	//bugkey
		DotLen = Speed[bug_spdvalue]*DotLength[bug_dotlenvalue]/10;
		DotSpLen = Speed[bug_spdvalue]*2-DotLen;
	}
	else{			//elekey
        DotLen = Speed[ele_spdvalue]*DotLength[ele_dotlenvalue]/10;
        DotSpLen = Speed[ele_spdvalue]*2-DotLen;
	}
}

//_̒ݒ肷
void DashLenSet(void)
{
	if(Mode ==3){		//elekey
    	DashLen = Speed[ele_spdvalue]*Ratio[ratiovalue]/10*DashLength[dashlenvalue]/10;
    	DashSpLen = Speed[ele_spdvalue]*2-Speed[ele_spdvalue]*DotLength[ele_dotlenvalue]/10;
	}
}

// C^vg JE^[ƃEFCNAbv
void __interrupt isr(void)
{   
    // Serial Received
    if(RCIF == 1){
        RCIF = 0;
        Recived();
    }
    
    // Timer2 interrupt 1mS
    if(TMR2IF == 1) {
    	TMR2IF = 0;
        
        if(scounter == 1){
        	mseconds++;
        	if(10000 < mseconds)
        		mseconds = 10000;
        }
        else
        	mseconds = 0;

        //LED _p 0.1bԓ_
        if(ledoffcounter == 1){
        	cseconds++;
        	if(100 < cseconds){
        		cseconds = 0;
        		LED_B = LED_OFF;
        		LED_Y = LED_OFF;
        		ledoffcounter = 0;
        	}
        }
        
        //X[v^C
        sleeptime++;
        if(SLEEP_TIME < sleeptime){
        	sleeptime = 0;
        	goto_sleep();
        }
        	
		DotSW 	  <<= 1;
		DashSW 	  <<= 1;
		ModeSW 	  <<= 1;
		DotLenSW  <<= 1;
		DashLenSW <<= 1;
		SpeedUPSW <<= 1;
		SpeedDNSW <<= 1;
		RatioSW   <<= 1;
		
		//eXCb`Ď
		DotSW 	  |= DotPort;
		DashSW    |= DashPort;
		ModeSW    |= ModePort;
		DotLenSW  |= DotLenPort;
		DashLenSW |= DashLenPort;
		SpeedUPSW |= SpeedUPPort;
		SpeedDNSW |= SpeedDNPort;
		RatioSW   |= RatioPort;

        //DotOn͒Z_oԁAwZ_KEY OŇAwZ_Xy[XKEY OFFɂ
        //DashOn͒_oԁAw蒷_KEY OŇAw蒷_Xy[XKEY OFFɂ
        if(Mode < 2){					//Through
       		if((DotSWClick == 1) || (DashSWClick == 1))	//Key ON
				KEYOUT = KEY_ON;
			else
				KEYOUT = KEY_OFF;
        }
        //oOL[
        else if(Mode == 2){
    		//P_
    		if(DotOn == 1){
      			if(0 < DashSpLen){			//_̎w莞ԃXy[Xo(_uo[΍)
          			DashSpLen--;
          			KEYOUT = KEY_OFF;
        		}
      			else if(0 < DotLen){		//Z_̎w莞ON
        			DotLen--;
          			KEYOUT = KEY_ON;
      			}
      			else{
      				if(0 < DotSpLen){		//Z_̎w莞ԑow莞ԃXy[Xo
          				DotSpLen--;
          				KEYOUT = KEY_OFF;
        			}
        			else{
          				DotOn = 0;
          			}
   				}
    		}
      		//_
      		else if(DashOn == 1){
               	if(0 < DotSpLen){
          			DotSpLen--;
          			KEYOUT = KEY_OFF;
        		}
               	if(DotSWClick == 1){		//_o͒ɒP_ꂽ(_uo[΍)
                	DotLenSet();
                	DashSpLen = DotSpLen;
					DotOn = 1;
					DashOn = 0;
      			}
      			else if(DashSWClick == 1)
       				KEYOUT = KEY_ON;
      			else{
       				KEYOUT = KEY_OFF;
                	//DotLenSet();			//_o͌シZ_ɈڍsZ_Ƃ̊Ԋu
                	//DashSpLen = DotSpLen;	//oOL[̓ȂȂ̂ŁÅԊu␳ɂƂǂ߂
                	DashSpLen = 10;			//̊Ԋu␳
   					DashOn = 0;
       			}
			}	
    		else{
      			//hbgꂽ̂ŒP_Ɉڂ
      			if(DotSWClick == 1){
					DotOn = 1;
					DotLenSet();
      			}
      			//_bVꂽ̂Œ_̐Ɉڂ
      			else if(DashSWClick == 1){
       				DashOn = 1;
      			}
    		}
		}
		//GL[
		else if(Mode == 3){
    		//P_
    		if(DotOn == 1){
   				if(DashSWClick == 1){		//Z_o͒ɒ_ꂽ
      				DashLenSet();
       			}
      			if(0 < DotLen){				//Z_̎w莞ON
        			DotLen--;
          			KEYOUT = KEY_ON;
      			}
      			else{
      				if(0 < DotSpLen){		//Z_̎w莞ԑow莞ԃXy[Xo
          				DotSpLen--;
          				KEYOUT = KEY_OFF;
        			}
        			else{
          				DotOn = 0;
          				if(0 < DashLen)
          					DashOn = 1;
          			}

   				}
    		}
      		//_
      		else if(DashOn == 1){
               	if(DotSWClick == 1){		//_o͒ɒP_ꂽ
                	DotLenSet();
                }
        		if(0 < DashLen){			//_̎w莞ON
          			DashLen--;
					KEYOUT = KEY_ON;
      			}
				else{
					if(0 < DashSpLen){		//_̎w莞ԑow莞ԃXy[Xo
          				DashSpLen--;
          				KEYOUT = KEY_OFF;
        			}
        			else{
          				DashOn = 0;
         				if(0 < DotLen)
          					DotOn = 1;
               		}
                }
			}	
    		else{
      			//hbgꂽ̂ŒP_Ɉڂ
      			if(DotSWClick == 1){
					DotOn = 1;
					DotLenSet();
      			}
      			//_bVꂽ̂Œ_̐Ɉڂ
      			if(DashSWClick == 1){
       				DashOn = 1;
					DashLenSet();
      			}
    		}
		}
    }
    
	// INT0 /RB0 Dash On
	if(INT0IF == 1) {
   		INT0IF = 0;
	}

	// INT1 RB1 TX
	if(INT1IF == 1) {
   		INT1IF = 0;
	}

	// INT2 RB2 ModeSW
	if(INT2IF == 1) {
  		INT2IF = 0;
	} 

	// RB port interrupt
	// RB4(RX),RB5(SPD DWN),RB6(DASHLEN),RB7(SPD UP)
	if(RBIF == 1) {
   		RBIF = 0;
	}
}

void main(void)
{ 
	int i;
	
	char swchk;
	static __bit ModeSWdone = 0;
	static __bit DotLenSWdone = 0;
	static __bit DashLenSWdone = 0;
	static __bit SpeedUPSWdone = 0;
	static __bit SpeedDNSWdone = 0;
	static __bit RatioSWdone = 0;
    
    // 8MHz OSCɐݒ
    OSCCONbits.IRCF = 0b111;
    
    // A/D converter module is disabled
    ADON = 0;       	// A/D converter module is disabled 
    PCFG0 = 1;      	// AN0 Pin configured as a digital port LED/Y
    PCFG1 = 1;      	// AN1 Pin configured as a digital port KEY OUT
    PCFG2 = 1;      	// AN2 Pin configured as a digital port DotLenPort
    PCFG3 = 1;      	// AN3 Pin configured as a digital port RatioPort
    PCFG4 = 1;      	// AN4 Pin configured as a digital port DashPort
    PCFG5 = 1;      	// AN5 Pin configured as a digital port TX
    PCFG6 = 1;      	// AN6 Pin configured as a digital port RX
    
    // input
    TRISB0 = 1;     	// Dash
    TRISB1 = 1;     	// TX
    TRISB2 = 1;     	// Dot
    TRISB3 = 1;     	// Mode
    TRISB4 = 1;     	// RX
    TRISB5 = 1;     	// Speed down
    TRISB6 = 1;     	// DashLen
    TRISB7 = 1;     	// Speed up
    TRISA2 = 1;     	// DotLenPort
    TRISA3 = 1;     	// RatioPort
   
    // output
    TRISA1 = 0;     	// KEY OUT
    TRISA0 = 0;     	// LED/Y
    TRISA7 = 0;     	// LED/B
    TRISA4 = 0;     	// not use
    TRISA6 = 0;     	// not use


    LATAbits.LA0 = 0;
	LATAbits.LA7 = 0;
	LATAbits.LA1 = 0;
	LATBbits.LB1 = 0;
	
    // pull-up
    RBPU = 0;               // PORTB pull-ups are enabled

    // interrupt
    // RB0-RB2
    INT0IF = 0;             // INT0 External Interrupt Flag bit Dash
    INT1IF = 0;             // INT1 External Interrupt Flag bit RX
    INT2IF = 0;             // INT2 External Interrupt Flag bit Dot
    INTEDG0 = 0;            // External Interrupt 0 Edge Select bit/0 = Interrupt on falling edge
    INTEDG1 = 0;            // External Interrupt 1 Edge Select bit/0 = Interrupt on falling edge
    INTEDG2 = 0;            // External Interrupt 2 Edge Select bit/0 = Interrupt on falling edge 
    // enable bit
    INT0IE = 1;             // INT0 External Interrupt Enable bit
    INT1IE = 0;             // INT1 External Interrupt Enable bit
    INT2IE = 1;             // INT2 External Interrupt Enable bit
    // priority
    INT1IP = 0;             // INT1 External Interrupt Priority bit
    INT2IP = 0;             // INT2 External Interrupt Priority bit
    RBIP = 0;               // RB Port Change Interrupt Priority bit 1=High priority
    TMR2IP = 1;
    
    // RB7:RB4
    RBIF = 0;               // PB Port Interrupt Flag bit UP DOWN
    RBIE = 1;               // Enables the RB port change interrupt

    // Timer2(8bit) use Fosc/4 (2000/16=125KHz 8uS)
    TMR2IF = 0;             // No TMR2 to PR2 match occurred
    T2CONbits.T2CKPS = 0b10;// Prescaler is 16
    PR2 = 125;              // Compare Value 1mS
    T2CONbits.TOUTPS = 0;   // Timer2 Output Postscale Select bits  
    TMR2IE = 1;             // timer2 
    TMR2ON = 1;           	// timer2 enable
    
    // EUSART setting
	// BRGH=0 and BRG16=1	16-bit/Asynchronous mode
	// Baud Rate = FOSC/[16(n+1)]
	// n=8000000/9600/16-1  =51
    SYNC = 0;				// 0 = Asynchronous mode
    SPEN = 1;				// 1 = Serial port enabled (configures RX/DT and TX/CK pins as serial port pins)
    CREN = 1;				// Continuous Receive Enable bit 1 = Enables receiver
    BRGH = 0;				// Asynchronous mode: 0 = Low speed
	BRG16 = 1;				// 1 = 16-bit Baud Rate Generator SPBRGH and SPBRG
   	SPBRGH  = 0;			// Baud Rate High Byte
   	SPBRG   = 51;			// Baud Rate Low Byte
    TXEN = 1;               // 1 = Transmit enabled
    RCIP = 0;               // EUSART Receive Interrupt Priority bit 1=High priority
    RCIF = 0;               // Receive Flag
    RCIE = 1;               // Enables the EUSART receive interrupt
    for(i=0; i < 20; i++)	// Receive Buffer Clear;
    	RcvBuf[i] = (unsigned char) 0;
    
    IPEN = 0;               // priority interrupts 荞ݗD斳
    PEIE = 0;               // Enables all low priority peripheral interrupts
    GIEH = 1;               // Enables all unmasked interrupts
    GIEL = 1;               // Enables all high priority interrupts
    GIE = 1;                //荞݊Jn
       
    // EEPROMϐf[^ݒ
	// addr0 0x10:ۑ ȊO:f[^ۑȂ
	// addr1 Mode (0..3)
	// addr2 Bugkey Speed (0..20)
	// addr3 Elekey Speed (0..20)
	// addr4 Bugkey DotLen(0..10)
	// addr5 Elekey DotLen(0..10)
	// addr6 Elekey Raito(0..9)
	// addr7 Elekey DashLen(0..10)
    if(EEPROM_read(0) == 0x10){
		Mode = EEPROM_read(1);
		bug_spdvalue = EEPROM_read(2);
		ele_spdvalue = EEPROM_read(3);
		bug_dotlenvalue = EEPROM_read(4);
		ele_dotlenvalue = EEPROM_read(5);
		ratiovalue = EEPROM_read(6);
		dashlenvalue = EEPROM_read(7);
	}
	else{
		set_default_value();
	}	

    while(1) {
        //-------------------------- DOT {^ -----------------------
        swchk = (char)(DotSW & 0b00000001);
        if(swchk == 1)
            DotSWClick = 0;
        if(swchk == 0)
            DotSWClick = 1;

        //-------------------------- DASH {^ -----------------------
        swchk = (char)(DashSW & 0b00000001);
        if(swchk == 1)
            DashSWClick = 0;
        if((swchk == 0) && (Mode != 0))
            DashSWClick = 1;

        //-------------------------- MODE {^ -----------------------
        //b`^Cv̏@1..0xFEԂ͖
        if(ModeSW == 0xFF)
            ModeSWClick = 0;
        if(ModeSW == 0)
            ModeSWClick = 1;
        
        if(ModeSWClick == 1){
			scounter = 1;			//Ă鎞Ԃv
        	if(ModeSWdone == 0){
        		Mode++;
                if(3 < Mode)
        			Mode = 0;
        		ModeSWdone = 1;
        		EEPROM_write(1, Mode);
       			LED_B_on();
        	}	
        	if(3000 < mseconds){
        		set_default_value();
        		scounter = 0;
       			LED_Y_on();
       			LED_B_on();
        	}
        }
        else{
        	ModeSWdone = 0;
			scounter = 0;
        }

        //----------------------- RATIO {^ -------------------------
        if(RatioSW == 0xFF)
            RatioSWClick = 0;
        if(RatioSW == 0)
            RatioSWClick = 1;
        
        //------------------------ DOT LEN {^ ----------------------
        if(DotLenSW == 0xFF)
            DotLenSWClick = 0;
        if(DotLenSW == 0)
            DotLenSWClick = 1;

        //----------------------- DASH LEN {^ ----------------------
        if(DashLenSW == 0xFF)
            DashLenSWClick = 0;
        if(DashLenSW == 0)
            DashLenSWClick = 1;

        //----------------------- SPD DN LEN {^ --------------------
        if(SpeedDNSW == 0xFF)
            SpeedDNSWClick = 0;
        if(SpeedDNSW == 0)
            SpeedDNSWClick = 1;

        //PƂŉꂽ
        if(RatioSWClick == 0 && DotLenSWClick == 0 && DashLenSWClick == 0){
        	if(SpeedDNSWClick == 1){
        		if(SpeedDNSWdone == 0){
        			if(Mode == 2){	//bug key
                    	if(bug_spdvalue < 20){
        					bug_spdvalue++;
   							EEPROM_write(2, bug_spdvalue);
       						LED_Y_on();
   						}
        			}
        			else if(Mode == 3){	//ele key
                    	if(ele_spdvalue < 20){
        					ele_spdvalue++;
   							EEPROM_write(3, ele_spdvalue);
       						LED_Y_on();
   						}
        			}
        			SpeedDNSWdone = 1;
        		}	
        	}
        	else{
        		SpeedDNSWdone = 0;
        	}
        }
        //RatioSW,DotLenSW,DashLenSŴǂꂩĂ
        else{
        	//Ratio
        	if(RatioSWClick == 1){
        		if(SpeedDNSWClick == 1){
        			if(SpeedDNSWdone == 0){
                   		if(0 < ratiovalue){
       						ratiovalue--;
       						SpeedDNSWdone = 1;
       						EEPROM_write(6, ratiovalue);
       						LED_Y_on();
       					}
       				}
       			}
        		else
        			SpeedDNSWdone = 0;
        	}

			//DotLen
        	if(DotLenSWClick == 1){
        		if(SpeedDNSWClick == 1){
        			if(Mode == 2){				//bug key
        				if(SpeedDNSWdone == 0){
                   			if(0 < bug_dotlenvalue){
        						bug_dotlenvalue--;
       							SpeedDNSWdone = 1;
        						EEPROM_write(4, bug_dotlenvalue);
	       						LED_Y_on();
        					}
        				}
        			}
        			else if(Mode == 3){			//ele key
        				if(SpeedDNSWdone == 0){
                   			if(0 < ele_dotlenvalue){
        						ele_dotlenvalue--;
       							SpeedDNSWdone = 1;
        						EEPROM_write(5, ele_dotlenvalue);
	       						LED_Y_on();
        					}
        				}
        			}
        		}
        		else
        			SpeedDNSWdone = 0;
        	}
        	
			//DashLen
        	if(DashLenSWClick == 1){
        		if(SpeedDNSWClick == 1){
        			if(Mode == 3){			//ele key
        				if(SpeedDNSWdone == 0){
                   			if(0 < dashlenvalue){
        						dashlenvalue--;
       							SpeedDNSWdone = 1;
   								EEPROM_write(7, dashlenvalue);
	       						LED_Y_on();
   							}
        				}
        			}
        		}
        		else
        			SpeedDNSWdone = 0;
        	}
        }
        
        //----------------------- SPD UP LEN {^ --------------------
        if(SpeedUPSW == 0xFF)
            SpeedUPSWClick = 0;
        if(SpeedUPSW == 0)
            SpeedUPSWClick = 1;

        //PƂŉꂽ
        if(RatioSWClick == 0 && DotLenSWClick == 0 && DashLenSWClick == 0){
        	if(SpeedUPSWClick == 1){
        		if(SpeedUPSWdone == 0){
        			if(Mode == 2){	//bug key
                    	if(0 < bug_spdvalue){
        					bug_spdvalue--;
   							EEPROM_write(2, bug_spdvalue);
       						LED_B_on();
   						}
        			}
        			else if(Mode == 3){	//ele key
                    	if(0 < ele_spdvalue){
        					ele_spdvalue--;
   							EEPROM_write(3, ele_spdvalue);
       						LED_B_on();
   						}
        			}
        			SpeedUPSWdone = 1;
        		}	
        	}
        	else{
        		SpeedUPSWdone = 0;
        	}
        }
        //RatioSW,DotLenSW,DashLenSŴǂꂩĂ
        else{
        	//Ratio
        	if(RatioSWClick == 1){
        		if(SpeedUPSWClick == 1){
        			if(SpeedUPSWdone == 0){
                   		if(ratiovalue < 9){
       						ratiovalue++;
       						SpeedUPSWdone = 1;
       						EEPROM_write(6, ratiovalue);
       						LED_B_on();
       					}
       				}
       			}
        		else
        			SpeedUPSWdone = 0;
        	}

			//DotLen
        	if(DotLenSWClick == 1){
        		if(SpeedUPSWClick == 1){
        			if(Mode == 2){				//bug key
        				if(SpeedUPSWdone == 0){
                   			if(bug_dotlenvalue < 10){
        						bug_dotlenvalue++;
       							SpeedUPSWdone = 1;
        						EEPROM_write(4, bug_dotlenvalue);
       							LED_B_on();
        					}
        				}
        			}
        			else if(Mode == 3){			//ele key
        				if(SpeedUPSWdone == 0){
                   			if(ele_dotlenvalue < 10)
        						ele_dotlenvalue++;
       						SpeedUPSWdone = 1;
        					EEPROM_write(5, ele_dotlenvalue);
       						LED_B_on();
        				}
        			}
        		}
        		else
        			SpeedUPSWdone = 0;
        	}
        	
			//DashLen
        	if(DashLenSWClick == 1){
        		if(SpeedUPSWClick == 1){
        			if(Mode == 3){			//ele key
        				if(SpeedUPSWdone == 0){
                   			if(dashlenvalue < 10){
        						dashlenvalue++;
       							SpeedUPSWdone = 1;
   								EEPROM_write(7, dashlenvalue);
	       						LED_B_on();
   							}
        				}
        			}
        		}
        		else
        			SpeedUPSWdone = 0;
        	}
        }
        
        //삵̂ŃX[v
        if(DotSWClick==1 || DashSWClick==1 || ModeSWClick==1 || DotLenSWClick==1
        	|| DashLenSWClick==1 || SpeedDNSWClick==1 || SpeedUPSWClick==1 || RatioSWClick==1)
        	sleeptime = 0;
        //--------------------  {^ CxgI --------------------   

    }

    return;
}
